import bs4
import requests
from cpl_query.extension import List
from discord.ext import tasks

from bot_core.logging.task_logger import TaskLogger
from modules.steam_special_offers.base.special_offer_watcher_abc import SpecialOfferWatcherABC
from modules.steam_special_offers.model.game_offer import GameOffer


class SteamOfferWatcher(SpecialOfferWatcherABC):
    def __init__(self, logger: TaskLogger):
        SpecialOfferWatcherABC.__init__(self)

        self._logger = logger

    def start(self):
        self.watch.start()

    def _get_max_count(self) -> int:
        count = 0
        result = requests.get(f"https://store.steampowered.com/search/results?specials=1")
        soup = bs4.BeautifulSoup(result.text, "lxml")
        element = soup.find_all("div", {"class": "search_results_count"})
        if len(element) < 1:
            return count

        count = int(element[0].contents[0].split(" ")[0].replace(",", ""))

        return count

    def _get_games_from_page(self, start: int, count: int) -> List[GameOffer]:
        games = List(GameOffer)
        result = requests.get(
            f"https://store.steampowered.com/search/results?query&start={start}&count={count}&force_infinite=1&specials=1"
        )
        soup = bs4.BeautifulSoup(result.text, "lxml")
        elements = soup.find_all("a", {"class": "search_result_row"})
        if len(elements) < 1:
            return games

        for element in elements:
            name_element = element.find("span", {"class": "title"})
            original_price_element = element.find("div", {"class": "discount_original_price"})
            discount_element = element.find("div", {"class": "discount_pct"})
            discount_price_element = element.find("div", {"class": "discount_final_price"})

            if (
                name_element is None
                or len(name_element.contents) < 1
                or original_price_element is None
                or len(original_price_element.contents) < 1
                or discount_element is None
                or len(discount_element.contents) < 1
                or discount_price_element is None
                or len(discount_price_element.contents) < 1
            ):
                continue

            name = name_element.contents[0]
            original_price = float(
                original_price_element.contents[0].replace(" ", "").replace("€", "").replace(",", ".")
            )
            discount = int(discount_element.contents[0].replace("%", ""))
            discount_price = float(
                discount_price_element.contents[0].replace(" ", "").replace("€", "").replace(",", ".")
            )
            games.add(GameOffer(name, original_price, discount_price, discount))

        return games

    async def _watch(self):
        self._logger.warn(__name__, "Watching")
        new_offers = List(GameOffer)

        max = self._get_max_count()
        for i in range(0, max + 100, 100):
            self._logger.debug(__name__, f"Get special offers from {i}")
            new_offers.extend(self._get_games_from_page(i, 100))

        self._logger.trace(__name__, "Finished watching")

    @tasks.loop(seconds=5)
    async def watch(self):
        try:
            await self._watch()
        except Exception as e:
            self._logger.error(__name__, f"Steam offer watcher failed", e)
